//===========================================================================
// Code for SaveListing.plw plugin
// It demonstrates how to get the disassembly lines for all addresses
//===========================================================================

// Bounden declarations
#define __NT__
#define __IDP__

#include <ida.hpp>
#include <idp.hpp>
#include <bytes.hpp>
#include <loader.hpp>
#include <kernwin.hpp>
#include <expr.hpp>

//===========================================================================
// init
//===========================================================================
int init(void) {
  return PLUGIN_OK;
}

//===========================================================================
// term
//===========================================================================
void term(void) {
}

//===========================================================================
// run
//===========================================================================
void run(int /*arg*/) {
  char *outfile = askfile_c(true, "*.lst", "Please specify the output file:");
  FILE *fp = fopen(outfile, "w");
  
  if(fp == NULL) {
    warning("Cannot create output file!");
    return;
  }

  //*** Structures
  
  // fprintf(fp, "==================================================================================================\n");
  // fprintf(fp, "\n");
  
  linearray_t linar(0);
    
  int i;
  int line_qty = 0;
  
  structplace_t st_pl(0, 0, 0);
  linar.set_place(&st_pl);
    
  while(true) {
    for(i = 0; i < linar.get_linecnt(); i++) {  // Process all of them
      char *line = linar.down();  // Get line
      char buf[MAXSTR];
      
      tag_remove(line, buf, sizeof(buf));  // Remove color codes
      
      // msg("%d: %s\n", i, buf);  // Display it on the message window
      fprintf(fp, "%s\n", buf);
      
      line_qty++;
    }
    
    if(line_qty%1000 == 0)
        msg("%d lines of structures done...\n", line_qty);
    
    st_pl.next(0);      
    
    if(st_pl.ending(0)) {
      char *line = linar.down();  // Get line
      char buf[MAXSTR];
      
      tag_remove(line, buf, sizeof(buf));  // Remove color codes
      
      // msg("%d: %s\n", i, buf);  // Display it on the message window
      fprintf(fp, "%s\n", buf);
      
      line_qty++;
    
      break;
    }
  }
    
  // fprintf(fp, "==================================================================================================\n");
  fprintf(fp, "\n");

  msg("\n%d lines of structures has written!\n\n", line_qty);
  
  //*** Code & data
  
  ea_t ea;
  ea_t min_ea = inf.minEA;
  ea_t max_ea = inf.maxEA;
  
  int n, flags;
  int j = 0;
  
  for(ea = min_ea; ea < max_ea && ea != BADADDR; ea = next_not_tail(ea)) {  // && ea < min_ea + 100;  
    idaplace_t pl;
    
    pl.ea = ea;
    pl.lnnum = 0;
    
    // User defined data for linearray_t: int *flag
    // *flag value is a combination of:
    // #define IDAPLACE_HEXDUMP 0x000F  // produce hex dump
    // #define IDAPLACE_STACK   0x0010  // produce 2/4/8 bytes per undefined item
    // (used to display the stack contents)
    // the number of displayed bytes depends on the stack bitness
    // not used yet because it confuses users:
    ////#define IDAPLACE_SHOWPRF 0x0020  // display line prefixes
    // #define IDAPLACE_SEGADDR 0x0040  // display line prefixes with the segment part 
    flags = calc_default_idaplace_flags();
    
    linearray_t ln(&flags);
    
    ln.set_place(&pl);
    
    // msg("Printing disassembly lines:\n");
    
    n = ln.get_linecnt();  // How many lines for this address?
    
    for(i = 0; i < n; i++) {  // Process all of them
      char *line = ln.down();  // Get line
      char buf[MAXSTR];
      
      tag_remove(line, buf, sizeof(buf));  // Remove color codes
      
      // msg("%d: %s\n", i, buf);  // Display it on the message window
      fprintf(fp, "%s\n", buf);
      
      j++;
    }
    
    if(j%1000 == 0)
        msg("%d lines done...\n", j);
  }
  
  msg("\n\n%d lines has written! Output file is done.\n", j);
  
  fclose(fp);
}

//===========================================================================

char comment[] = "Generate disassembly lines for all addresses";
char help[] = "Generate disassembly lines for all addresses\n";

//===========================================================================

// This is the preferred name of the plugin module in the menu system
// The preferred name may be overridden in plugins.cfg file

char wanted_name[] = "Save the Listing";

//===========================================================================

// This is the preferred hotkey for the plugin module
// The preferred hotkey may be overriden in plugins.cfg file
// Note: IDA won't tell you if the hotkey is not correct 
// It will just disable the hotkey.

char wanted_hotkey[] = "";

//===========================================================================
// PLUGIN DESCRIPTION BLOCK
//===========================================================================
plugin_t PLUGIN =
{
  IDP_INTERFACE_VERSION,
  0,  // Plugin flags
  init,  // Initialize
  term,  // Terminate. this pointer may be NULL
  run,  // Invoke plugin
  comment,  // Long comment about the plugin it could appear in the status line or as a hint
  help,  // Multiline help about the plugin
  wanted_name,  // The preferred short name of the plugin
  wanted_hotkey  // The preferred hotkey to run the plugin
};

//===========================================================================
//===========================================================================
